/*
 * Simple demo program, extremely minimalist, to show FreeRTOS™ v10
 * also running on an Arduno UNO using static allocation.
 * 
 * 
 * PLEASE NOTE: To compile, You MUST set ...
 * #define configSUPPORT_STATIC_ALLOCATION     1
 * ...in FreeRTOSConfig.h
 * 
 * 
 *                  Guglielmo Braguglia for
 *   Arduino Day 2018 - Futura Elettronica - Gallarate (VA)
 * 
 */
 
#include <Arduino_FreeRTOS.h>
#include <task.h>
#include <timers.h>

StackType_t   uxStackBuffer_T1[192];
StaticTask_t  xTaskBuffer_T1;
StackType_t   uxStackBuffer_T2[192];
StaticTask_t  xTaskBuffer_T2;

StaticTimer_t xTimerBuffer;
TimerHandle_t xTimers;

byte          LEDstatus = LOW;

void MyTask1(void * pvParameters) {
   for (;;) {
      Serial.println("Message from task1");
      vTaskDelay(2500 / portTICK_PERIOD_MS);
   }
}

void MyTask2(void * pvParameters) {
   for (;;) {
      Serial.println("Message from task2");
      vTaskDelay(5000 / portTICK_PERIOD_MS);
   }
}

void vTimer1(TimerHandle_t xTimer) {
   LEDstatus = ~LEDstatus;
   digitalWrite(LED_BUILTIN, LEDstatus);
}

void setup() {
   Serial.begin(115200);
   pinMode(LED_BUILTIN, OUTPUT);
   digitalWrite(LED_BUILTIN, LEDstatus);
   Serial.println("Programa Start ...");
   //
   xTaskCreateStatic(MyTask1, "Task1", 192, NULL, 1, uxStackBuffer_T1, &xTaskBuffer_T1);
   xTaskCreateStatic(MyTask2, "Task2", 192, NULL, 1, uxStackBuffer_T2, &xTaskBuffer_T2);
   Serial.println("... tasks created");
   //
   xTimers = xTimerCreateStatic("Timer1", pdMS_TO_TICKS(1000), pdTRUE, (void *) 1, vTimer1, &xTimerBuffer);
   if (xTimers != NULL) {
      xTimerStart(xTimers, 0);
      Serial.println("... timer created");
   }
   else Serial.println("... unable to create timer");
   //
   Serial.println();
}

void loop() {
   // DO Nothing Here !
}

/*
   -------------------------------------------------------------------------------
                              DO NOT MODIFY FROM HERE
   -------------------------------------------------------------------------------
*/

#if configSUPPORT_STATIC_ALLOCATION == 1
/* configSUPPORT_STATIC_ALLOCATION is set to 1, so the application must provide an
   implementation of vApplicationGetIdleTaskMemory() to provide the memory that is
   used by the Idle task. */
void vApplicationGetIdleTaskMemory( StaticTask_t ** ppxIdleTaskTCBBuffer,
                                    StackType_t ** ppxIdleTaskStackBuffer,
                                    uint32_t * pulIdleTaskStackSize ) {
   /* If the buffers to be provided to the Idle task are declared inside this
      function then they must be declared static - otherwise they will be allocated on
      the stack and so not exists after this function exits. */
   static StaticTask_t xIdleTaskTCB;
   static StackType_t uxIdleTaskStack[ configMINIMAL_STACK_SIZE ];

   /* Pass out a pointer to the StaticTask_t structure in which the Idle task's
      state will be stored. */
   *ppxIdleTaskTCBBuffer = &xIdleTaskTCB;

   /* Pass out the array that will be used as the Idle task's stack. */
   *ppxIdleTaskStackBuffer = uxIdleTaskStack;

   /* Pass out the size of the array pointed to by *ppxIdleTaskStackBuffer.
      Note that, as the array is necessarily of type StackType_t,
      configMINIMAL_STACK_SIZE is specified in words, not bytes. */
   *pulIdleTaskStackSize = configMINIMAL_STACK_SIZE;
}
/*-----------------------------------------------------------*/

/* configSUPPORT_STATIC_ALLOCATION and configUSE_TIMERS are both set to 1, so the
   application must provide an implementation of vApplicationGetTimerTaskMemory()
   to provide the memory that is used by the Timer service task. */
void vApplicationGetTimerTaskMemory( StaticTask_t ** ppxTimerTaskTCBBuffer,
                                     StackType_t ** ppxTimerTaskStackBuffer,
                                     uint32_t * pulTimerTaskStackSize ) {
   /* If the buffers to be provided to the Timer task are declared inside this
      function then they must be declared static - otherwise they will be allocated on
      the stack and so not exists after this function exits. */
   static StaticTask_t xTimerTaskTCB;
   static StackType_t uxTimerTaskStack[ configTIMER_TASK_STACK_DEPTH ];

   /* Pass out a pointer to the StaticTask_t structure in which the Timer
      task's state will be stored. */
   *ppxTimerTaskTCBBuffer = &xTimerTaskTCB;

   /* Pass out the array that will be used as the Timer task's stack. */
   *ppxTimerTaskStackBuffer = uxTimerTaskStack;

   /* Pass out the size of the array pointed to by *ppxTimerTaskStackBuffer.
      Note that, as the array is necessarily of type StackType_t,
      configTIMER_TASK_STACK_DEPTH is specified in words, not bytes. */
   *pulTimerTaskStackSize = configTIMER_TASK_STACK_DEPTH;
}

#endif
